# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation; either version 2 of the License, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
""" Copyright (c) 2002-2003 LOGILAB S.A. (Paris, FRANCE).
 http://www.logilab.fr/ -- mailto:contact@logilab.fr

 Layout objects
"""

__revision__ = "$Id: layouts.py,v 1.2 2003/10/31 08:04:39 syt Exp $"

from logilab.common.tree import VNode


class UnalignmentError(Exception):
    """ raised when all lines of a table doesn't have the same length
    """

class SectionLayout(VNode):
    """ a section is caracterized by a title and a description
    """
    def __init__(self, title, description = ''):
        VNode.__init__(self, title = title)
        self.description = description


class TableLayout(VNode):
    """ a table of data (list of list of strings)
    """
    
    def __init__(self, data, row_headers = 0, col_headers = 0):
        """
        data : list of list containing table's data
        row_headers : if true, the first row's cells will be considered as
                      headers
        col_headers : if true, the first column's cells'll be considered as
                      headers
        """
        VNode.__init__(self)
        self.data = data
        self.row_headers = row_headers
        self.col_headers = col_headers
        # check table
        result = [0]*len(self.data[0])
        for row in self.data:
            for index in range(len(row)):
                col = row[index]
                try:
                    result[index] = max(result[index], len(col))
                except IndexError:
                    raise UnalignmentError()
                except TypeError:
                    raise Exception('Table cell\'s value must be a string')
                    
        self._columns_size = result
        
    def get_columns_size(self):
        """ return a list with each column size """
        return self._columns_size
        
        
class VerbatimLayout(VNode):
    """ a verbatim layout, display the raw data
    """
    def __init__(self, verbatim_string):
        VNode.__init__(self)
        self.data = verbatim_string
