# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation; either version 2 of the License, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
""" Copyright (c) 2000-2003 LOGILAB S.A. (Paris, FRANCE).
 http://www.logilab.fr/ -- mailto:contact@logilab.fr

utilities methods and classes for reporters
"""

__revision__ = "$Id: __init__.py,v 1.7 2003/11/19 11:28:47 syt Exp $"

import sys

CMPS = ['=', '-', '+']

def diff_string(old, new):
    """given a old and new int value, return a string representing the
    difference
    """
    try:
        diff = abs(old - new)
        diff_str = "%s%s" % (CMPS[old.__cmp__(new)], diff or '')
    except Exception, ex:
        # the int.__cmp__ doesn't work with python < 2.2
        diff_str = str(ex)
    return diff_str


class BaseReporter:
    """base class for reporters"""
    
    def __init__(self, output=sys.stdout):
        self.include_ids = None
        self.section = 0
        # output stream
        self.out = output
        
    def writeln(self, string=''):
        """write a line in the output buffer
        """
        print >> self.out, string

    def write(self, string=''):
        """write a line in the output buffer
        """
        print >> self.out, string,
        
    def display_results(self, layout):
        """display results encapsulated in the layout tree
        """
        self.section = 0
        self._display(layout)
        
    def _display(self, layout):
        """recurse on the layout tree and call their accept method
        (see the Visitor pattern)
        """
        if self.include_ids and hasattr(layout, 'report_id'):
            layout.title = '%s (%s)' % (layout.title, layout.report_id)
        layout.accept(self)
        for child in layout.children:
            self._display(child)
        try:
            layout.accept(self, close=1)
        except TypeError:
            return
        
    def visit_sectionlayout(self, layout, close=0):
        """maintain a section level counter"""
        if close:
            self.section -= 1
        else:
            self.section += 1

        
